@@ -33,16 +33,11 @@
 #include <libgnome/gnome-init.h>
 
 #include "yelp-utils.h"
+#include "yelp-debug.h"
 
 #include <string.h>
 
-#ifdef YELP_DEBUG
-#define d(x) x
-#else
-#define d(x)
-#endif
-
-GHashTable *doc_info_table;
+static GHashTable *doc_info_table;
 
 typedef struct {
     gchar       *uri;
@@ -68,6 +63,23 @@ struct _YelpDocInfo {
     gint ref_count;
 };
 
+static gchar *mandirs[] = {
+    "man0p",
+    "man1",
+    "man1p",
+    "man2",
+    "man3",
+    "man3p",
+    "man4",
+    "man5",
+    "man6",
+    "man7",
+    "man8",
+    "man9",
+    "mann",
+    NULL
+};
+
 static YelpDocType  get_doc_type       (gchar   *uri);
 static gchar *      convert_ghelp_uri  (gchar   *uri);
 
@@ -76,6 +88,12 @@ static gchar *      convert_info_uri   (gchar   *uri);
 
 static gchar *dot_dir = NULL;
 
+static gchar **infopath = NULL;
+static gchar *infopath_d[] = {"/usr/info", "/usr/share/info", 
+			      "/usr/local/info", "/usr/local/share/info", 
+			      NULL};
+
+
 const char *
 yelp_dot_dir (void)
 {
@@ -109,8 +127,8 @@ yelp_doc_info_new (const gchar *uri, gboolean trust_uri)
 
     g_return_val_if_fail (uri != NULL, NULL);
 
-    d (g_print ("yelp_doc_info_new\n"));
-    d (g_print ("  uri      = \"%s\"\n", uri));
+    debug_print (DB_FUNCTION, "entering\n");
+    debug_print (DB_ARG, "  uri      = \"%s\"\n", uri);
 
     if (trust_uri)
 	full_uri = gnome_vfs_make_uri_from_input (uri);
@@ -141,8 +159,18 @@ yelp_doc_info_new (const gchar *uri, gboolean trust_uri)
     }
     else if (g_str_has_prefix (full_uri, "info:")) {
 	doc_uri  = convert_info_uri (full_uri);
-	doc_type = YELP_DOC_TYPE_INFO;
-	uri_type = YELP_URI_TYPE_INFO;
+	if (!g_str_has_prefix (doc_uri, "man:")) {
+	    doc_type = YELP_DOC_TYPE_INFO;
+	    uri_type = YELP_URI_TYPE_INFO;
+	} else {
+	    gchar *tmp;
+	    tmp = g_strdup (doc_uri);
+	    g_free (doc_uri);
+	    doc_uri = convert_man_uri (tmp, trust_uri);
+	    g_free (tmp);
+	    doc_type = YELP_DOC_TYPE_MAN;
+	    uri_type = YELP_URI_TYPE_MAN;	    
+	}
     }
     else if (g_str_has_prefix (full_uri, "x-yelp-toc:")) {
 	doc_uri = g_strdup ("file://" DATADIR "/yelp/toc.xml");
@@ -159,9 +187,9 @@ yelp_doc_info_new (const gchar *uri, gboolean trust_uri)
 	doc_type = YELP_DOC_TYPE_EXTERNAL;
 	uri_type = YELP_URI_TYPE_EXTERNAL;
     }
-
-    d (g_print ("  full_uri = \"%s\"\n", full_uri));
-    d (g_print ("  doc_uri  = \"%s\"\n", doc_uri));
+    
+    debug_print (DB_ARG, "  full_uri = \"%s\"\n", full_uri);
+    debug_print (DB_ARG, "  doc_uri  = \"%s\"\n", doc_uri);
 
     g_free (full_uri);
 
@@ -193,8 +221,8 @@ yelp_doc_info_get (const gchar *uri, gboolean trust_uri)
 
     g_return_val_if_fail (uri != NULL, NULL);
 
-    d (g_print ("yelp_doc_info_get\n"));
-    d (g_print ("  uri     = \"%s\"\n", uri));
+    debug_print (DB_FUNCTION, "entering\n");
+    debug_print (DB_ARG, "  uri     = \"%s\"\n", uri);
 
     if (!doc_info_table)
 	doc_info_table =
@@ -215,7 +243,7 @@ yelp_doc_info_get (const gchar *uri, gboolean trust_uri)
     doc = (YelpDocInfo *) g_hash_table_lookup (doc_info_table, doc_uri);
 
     if (!doc) {
-	doc = yelp_doc_info_new (doc_uri, trust_uri);
+	doc = yelp_doc_info_new (uri, trust_uri);
 	if (doc && doc->type != YELP_DOC_TYPE_EXTERNAL) {
 	    YelpDocInfo *old_doc = NULL;
 
@@ -273,8 +301,8 @@ yelp_doc_info_free (YelpDocInfo *doc)
 {
     gint i;
 
-    d (g_print ("yelp_doc_info_free\n"));
-    d (g_print ("  uri = \"%s\"\n", doc->uris->uri));
+    debug_print (DB_FUNCTION, "entering\n");
+    debug_print (DB_ARG, "  uri = \"%s\"\n", doc->uris->uri);
 
     if (!doc)
 	return;
@@ -551,7 +579,7 @@ get_doc_type (gchar *uri)
     if (mime_type == NULL)
 	return YELP_DOC_TYPE_ERROR;
 
-    if (g_str_equal (mime_type, "text/xml"))
+    if (g_str_equal (mime_type, "text/xml") || g_str_equal (mime_type, "application/docbook+xml") || g_str_equal (mime_type, "application/xml"))
 	type = YELP_DOC_TYPE_DOCBOOK_XML;
     else if (g_str_equal (mime_type, "text/sgml"))
 	type = YELP_DOC_TYPE_DOCBOOK_SGML;
@@ -573,7 +601,7 @@ yelp_doc_info_add_uri (YelpDocInfo *doc_info,
 {
     DocInfoURI *info_uri;
 
-    d (g_print ("yelp_doc_add_uri\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     g_assert (doc_info->num_uris <= doc_info->max_uris);
 
@@ -591,9 +619,9 @@ yelp_doc_info_add_uri (YelpDocInfo *doc_info,
 
     doc_info->num_uris++;
 
-    d (g_print ("  uri      = \"%s\"\n", uri));
-    d (g_print ("  num_uris = %i\n", doc_info->num_uris));
-    d (g_print ("  max_uris = %i\n", doc_info->max_uris));
+    debug_print (DB_ARG, "  uri      = \"%s\"\n", uri);
+    debug_print (DB_ARG, "  num_uris = %i\n", doc_info->num_uris);
+    debug_print (DB_ARG, "  max_uris = %i\n", doc_info->max_uris);
 }
 
 /******************************************************************************/
@@ -836,37 +864,9 @@ convert_man_uri (gchar *uri, gboolean trust_uri)
     return doc_uri;
 }
 
-static gchar *
-convert_info_uri (gchar   *uri)
+gchar **
+yelp_get_info_paths ( )
 {
-    gchar *path, *cur;
-    gchar *doc_uri  = NULL;
-    gchar *info_name = NULL;
-    gchar *info_dot_info = NULL;
-
-    static gchar **infopath = NULL;
-    static gchar *infopath_d[] = {"/usr/info", "/usr/share/info", NULL};
-
-    gint i;
-
-    GDir  *dir;
-    const gchar *filename;
-    gchar *pattern;
-    GPatternSpec *pspec = NULL;
-    GPatternSpec *pspec1 = NULL;
-
-    if ((path = strchr(uri, ':')))
-	path++;
-    else
-	goto done;
-
-    /* An absolute path after info: */
-    if (path[0] == '/') {
-	if (g_file_test (path, G_FILE_TEST_IS_REGULAR))
-	    doc_uri = g_strconcat ("file://", path, NULL);
-	goto done;
-    }
-
     /* Get the infopath, either from the INFOPATH envar,
        or from the default infopath_d.
     */
@@ -883,6 +883,49 @@ convert_info_uri (gchar   *uri)
 	}
     }
 
+
+    return infopath;
+}
+
+
+gchar **
+yelp_get_man_paths ()
+{
+	return mandirs;
+}
+
+static gchar *
+convert_info_uri (gchar   *uri)
+{
+    gchar *path, *cur;
+    gchar *doc_uri  = NULL;
+    gchar *info_name = NULL;
+    gchar *info_dot_info = NULL;
+    gchar **infopaths = NULL;
+    gboolean need_subdir = FALSE;
+
+    gint i;
+
+    GDir  *dir;
+    const gchar *filename;
+    gchar *pattern;
+    GPatternSpec *pspec = NULL;
+    GPatternSpec *pspec1 = NULL;
+    gchar *subdir = NULL;
+
+    if ((path = strchr(uri, ':')))
+	path++;
+    else
+	goto done;
+
+    /* An absolute path after info: */
+    if (path[0] == '/') {
+	if (g_file_test (path, G_FILE_TEST_IS_REGULAR))
+	    doc_uri = g_strconcat ("file://", path, NULL);
+	goto done;
+    }
+
+
     /* The URI is one of the following:
        info:info_name
        info:info_name#node
@@ -905,6 +948,20 @@ convert_info_uri (gchar   *uri)
     else
 	info_name = g_strdup (path);
 
+    if (strstr (info_name, "/")) {
+	gchar *tmp = NULL;
+	gchar *real_name = NULL;
+	tmp = strstr (info_name, "/");
+	tmp++;
+	real_name = g_strdup (tmp);
+	subdir = g_strndup (info_name, (strstr (info_name, "/") - info_name));
+	g_free (info_name);
+	info_name = g_strdup (real_name);
+	g_free (real_name);
+	need_subdir = TRUE;
+    }
+
+
     pattern = g_strdup_printf ("%s.info.*", info_name);
     pspec = g_pattern_spec_new (pattern);
     g_free (pattern);
@@ -914,18 +971,38 @@ convert_info_uri (gchar   *uri)
 
     info_dot_info = g_strconcat (info_name, ".info", NULL);
 
-    for (i = 0; infopath[i]; i++) {
+    infopaths = yelp_get_info_paths ();
+
+    for (i = 0; infopaths[i]; i++) {
 	dir = g_dir_open (infopath[i], 0, NULL);
 	if (dir) {
 	    while ((filename = g_dir_read_name (dir))) {
-		if (g_str_equal (info_dot_info, filename)  ||
-		    g_pattern_match_string (pspec, filename) ||
-		    g_pattern_match_string (pspec1, filename) ||
-		    g_str_equal (info_name, filename)) {
-		    doc_uri = g_strconcat ("file://",
-					   infopath[i], "/",
-					   filename,
-					   NULL);
+		if (need_subdir && g_str_equal (filename, subdir)) {
+		    gchar *dirname = NULL;
+		    g_dir_close (dir);
+		    dirname = g_strconcat (infopath[i], "/", subdir, NULL);
+		    dir = g_dir_open (dirname, 0, NULL);
+		    g_free (dirname);
+		    filename = g_dir_read_name (dir);
+		    need_subdir = FALSE;
+		}
+		else if (!need_subdir && 
+			 (g_str_equal (info_dot_info, filename)  ||
+			  g_pattern_match_string (pspec, filename) ||
+			  g_pattern_match_string (pspec1, filename) ||
+			  g_str_equal (info_name, filename))) {
+		    if (subdir) {
+			doc_uri = g_strconcat ("file://",
+					       infopath[i], "/", subdir, "/",
+					       filename,
+					       NULL);
+		    } else {
+			doc_uri = g_strconcat ("file://",
+					       infopath[i], "/",
+					       filename,
+					       NULL);
+
+		    }
 		    g_dir_close (dir);
 		    goto done;
 		}
@@ -934,12 +1011,23 @@ convert_info_uri (gchar   *uri)
 	}
     }
 
+    /* If we got this far and doc_uri is still NULL, we resort to looking
+     * for a man page.  Let above handle that.  We just let it know that 
+     * somethings gotta be done */
+    if (doc_uri == NULL) {
+	gchar *tmp;
+	tmp = strchr (uri, ':');
+	doc_uri = g_strconcat ("man",tmp,NULL);
+    }
+
  done:
     if (pspec)
 	g_pattern_spec_free (pspec);
     if (pspec1)
 	g_pattern_spec_free (pspec1);
+    g_free (subdir);
     g_free (info_dot_info);
     g_free (info_name);
+    
     return doc_uri;
 }
